#ifndef __CLIENT_ACTIONS_H_
#define __CLIENT_ACTIONS_H_

#include "action_map.h"
#include "communication.h"

namespace te
{
//#############################################################################
class action_exit : public action
{
public: //---------------------------------------------------------------------
    action_exit(commLink& its_out_comm) : my_out_comm_(its_out_comm)
    {
        dgram_.command_ = exit;
    }
//-----------------------------------------------------------------------------
    void invoke()
    {
        my_out_comm_.dispatch(dgram_);
    }
private:
    commLink& my_out_comm_;
    datagram  dgram_;
};
//#############################################################################
class action_map_make : public action
{
public: //---------------------------------------------------------------------
    action_map_make (commLink& its_out_comm) : my_out_comm_(its_out_comm)
    {
        dgram_.command_ = map_make;
        dgram_.params_.resize(2);
    }
//-----------------------------------------------------------------------------
    void setSize(size_t size_long, size_t size_latt)
    {
        dgram_.params_[0] = size_long;
        dgram_.params_[1] = size_latt;
    }
//-----------------------------------------------------------------------------
    void invoke()
    {
        my_out_comm_.dispatch(dgram_);
    }
private: //--------------------------------------------------------------------
    commLink& my_out_comm_;
    datagram  dgram_;
};
//#############################################################################
class action_area_req : public action
{
public: //---------------------------------------------------------------------
    action_area_req (commLink& its_out_comm) : my_out_comm_(its_out_comm)
    {
        dgram_.command_ = area_req;
        dgram_.params_.resize(0);
    }
//-----------------------------------------------------------------------------
    void setArea(size_t lower_left_long,
                 size_t lower_left_latt,
                 size_t upper_right_long,
                 size_t upper_right_latt)
    {
        dgram_.params_.push_back(lower_left_long);
        dgram_.params_.push_back(lower_left_latt);
        dgram_.params_.push_back(upper_right_long);
        dgram_.params_.push_back(upper_right_latt);
    }
//-----------------------------------------------------------------------------
    void addVersion(size_t leaf_version)
    {
        dgram_.params_.push_back(leaf_version);
    }
//-----------------------------------------------------------------------------
    void invoke()
    {
        my_out_comm_.dispatch(dgram_);
        dgram_.params_.resize(0);
    }
private: //--------------------------------------------------------------------
    commLink& my_out_comm_;
    datagram  dgram_;
};
//#############################################################################

};
#endif