/*
* Adam Kadlubek 2006
* Terrain Generator for TTD
* under GPL license
* version written in C, not protable
* to C++
*
* This is actually a refactor of a C++ class
* that is why all ex-methods have the explicit
* 'this_' pointer
*/

#include "gridArray.h"

#ifdef TTD
#include "cwrap.h"
#include <stdint.h>
#include <ttdvar.h>
extern uint32_t (*randomfn)(void) asm("randomfn");
#endif

#ifdef TE
#include <iostream>
using namespace te;
uint32_t climate;
#endif


/*
*
* Snippet constructor - creates an array as a sub-array of another array
*
*/

void snipArray (gridArrayPtr source_,ulong size_x_,ulong size_y_,gridArrayPtr* this_) {

    ulong limit_x = 0;
    ulong limit_y = 0;
    ulong i_x = 0;
    ulong i_y = 0;

    /* check if the requested sizes are fitting to the
       size of the array used */

    if (size_x_ > size_x(source_))
    {
	    limit_x = size_x(source_);
    } else {
	    limit_x = size_x_;
    }

    if (size_y_ > size_y(source_))
    {
      limit_y = size_y(source_);
    } else {
      limit_y = size_y_;
    }

    makeArray(limit_x,limit_y,NULL,this_);  

    for (i_x = 0; i_x < limit_x; ++i_x)
    {
        for (i_y = 0; i_y < limit_y; ++i_y)
        {
            (*this_)->data[i_x][i_y] = val(i_x,i_y,source_);
        }
    }

    (*this_)->size_x = limit_x;
    (*this_)->size_y = limit_y;
#ifdef TE
    (*this_)->thSpeci1 = source_->thSpeci1;
    (*this_)->thSpeci2 = source_->thSpeci2;
#endif
}

/*
*
* new constructor - creates an array of a given size
* and fill it with random data
*
*/

void makeArray (ulong size_x_,ulong size_y_,
                uint32_t (*randomizer)(void),gridArrayPtr* this_)
{

/* first check if the pointer we get is NULL or not */

    ulong i_x;
    ulong i_y;

    if ((*this_) != NULL)
    {
        /* freeing memory */
        for (i_x = 0;i_x < size_x(*this_);++i_x)
            {
                free((*this_)->data[i_x]);
            }
        free ((*this_)->data);
#ifdef  TE
        delete(*this_);        
#else  if
        free (*this_);
#endif
    }

    /* getting memory */
#ifdef TE
    *this_ = new gridArray;
#else if
    (*this_) = (gridArrayPtr)malloc(sizeof(gridArray));
#endif
    (*this_)->data = (float**)malloc(sizeof(float*)*size_x_);

    for (i_x = 0;i_x < size_x_;++i_x)
        {
           (*this_)->data[i_x] = (float*)malloc(sizeof(float)*size_y_);
        }

    /* done */

    for (i_x = 0;i_x < size_x_; ++i_x)
    {
        for (i_y = 0;i_y < size_y_; ++i_y)
        {
            if (randomizer != NULL)
            (*this_)->data[i_x][i_y] = (randomizer()*1.0f/0xffffffff);
            else
            (*this_)->data[i_x][i_y] = 0.0f;
        }
    }
    (*this_)->size_x = size_x_;
    (*this_)->size_y = size_y_;

#ifdef TE
    (*this_)->thSpeci1 = 0;
    (*this_)->thSpeci2 = 0;
#endif


}

/*
*
* add elements of array b to 'this' array
*
*/


void addArray(gridArrayPtr source_,gridArrayPtr this_) {

    ulong limit_x = 0;
    ulong limit_y = 0;
    ulong i_x = 0;
    ulong i_y = 0;

    // check if the array sizes match
    if (size_x(this_) != size_x(source_) && size_y(this_) != size_y(source_))
    {
        return;
    }

    // do the addition

    limit_x = size_x(this_);
    limit_y = size_y(this_);
 
    for (i_x = 0;i_x < limit_x;++i_x)
    {
        for (i_y = 0;i_y < limit_y;++i_y)
        {
            this_->data[i_x][i_y] += val(i_x,i_y,source_);
        }
    }
}

/*
*
* add an value of b to all array elements
*
*/

void addScalar (float scalar, gridArrayPtr this_) {

    ulong limit_x = size_x(this_);
    ulong limit_y = size_y(this_);
    ulong i_x = 0;
    ulong i_y = 0;

    for (i_x = 0;i_x < limit_x;++i_x)
    {
        for (i_y = 0;i_y < limit_y;++i_y)
        {
            this_->data[i_x][i_y] += scalar;
            //insert(1.0,i_x,i_y,this_);
        }
    }
}

/*
*
* multiply the array with a scalar
*
*/

void mulScalar(float scalar, gridArrayPtr this_) {

    ulong limit_x = size_x(this_);
    ulong limit_y = size_y(this_);
    ulong i_x = 0;
    ulong i_y = 0;

    for (i_x = 0;i_x < limit_x;++i_x)
    {
        for (i_y = 0;i_y < limit_y;++i_y)
        {
            this_->data[i_x][i_y] *= scalar;
        }
    }
}

/*
*
* scaling(enlarging) of the image to the x-y
* it is just a pixel resize, to get filtering use filter
* method
*
*/

void scale(ulong newSize_x,ulong newSize_y, gridArrayPtr* this_) {

    ulong indiceBuf_x = 0;
    ulong indiceBuf_y = 0;
    ulong i_x = 0;
    ulong i_y = 0;
    gridArrayPtr buffer = NULL;
    gridArrayPtr tmp;
    float tmp2;

    if (newSize_x < size_x(*this_) || newSize_y < size_y(*this_))
        return;

    makeArray(newSize_x,newSize_y,NULL,&buffer); // new, resized array

    while (indiceBuf_x < newSize_x)
    {
          
        if(!dist(newSize_x,indiceBuf_x,size_x(*this_),i_x,*this_))
        {
            ++i_x;
        }
        while (indiceBuf_y < newSize_y)
        {
            if(!dist(newSize_y,indiceBuf_y,size_y(*this_),i_y,*this_))
            {
                ++i_y;
            }
            tmp2 = val(i_x,i_y,(*this_));
            insert(tmp2,indiceBuf_x,indiceBuf_y,buffer);
            indiceBuf_y++;
        }
        indiceBuf_y = 0;
	      i_y = 0;
        ++indiceBuf_x;
    }

    /* the painful part */
    tmp = *this_;
    *this_ = buffer;
    buffer = tmp;
    destroyArray(&buffer);
}

/*
*
* Bilinear filtering of the image
* Ironically - this is the longest stage
*
*/

void filter(ulong iterations, gridArrayPtr* this_) {

    gridArrayPtr buffer = NULL;
    gridArrayPtr tmp = NULL;
    float value;

    ulong limit_x = size_x(*this_) - 1;
    ulong limit_y = size_y(*this_) - 1;
    ulong i_x;
    ulong i_y;
    ulong iter;

    makeArray(size_x(*this_),size_y(*this_),NULL,&buffer);

    for (iter = 0;iter < iterations;++iter) {

        // first stage - body of the heightmap

        for (i_x = 1;i_x < limit_x;++i_x)
        {
            for (i_y = 1;i_y < limit_y;++i_y)
            {              
                value = val(i_x  ,i_y  ,*this_) * 0.5f   +
                        val(i_x+1,i_y  ,*this_) * 0.125f +
                        val(i_x-1,i_y  ,*this_) * 0.125f +
                        val(i_x  ,i_y+1,*this_) * 0.125f +
                        val(i_x  ,i_y-1,*this_) * 0.125f;

                insert(value,i_x,i_y,buffer);
            }
        }

/* edges */

        for (i_x = 1;i_x < limit_x;++i_x) {

                value = val(i_x  ,0,*this_) * 0.5f   +
                        val(i_x+1,0,*this_) * 0.125f +
                        val(i_x-1,0,*this_) * 0.125f +
                        val(i_x  ,1,*this_) * 0.125f;

                insert(value,i_x,0,buffer);

                value = val(i_x  ,limit_y  ,*this_) * 0.5f   +
                        val(i_x+1,limit_y  ,*this_) * 0.125f + 
                        val(i_x-1,limit_y  ,*this_) * 0.125f +
                        val(i_x  ,limit_y-1,*this_) * 0.125f;

                insert(value,i_x,limit_y,buffer);
           }

        /* now edges along y */
       

        for(i_y = 1;i_y < limit_y;++i_y) {

                value = val(0,i_y  ,*this_) * 0.5f   +
                        val(0,i_y+1,*this_) * 0.125f +
                        val(0,i_y-1,*this_) * 0.125f +
                        val(1,i_y  ,*this_) * 0.125f;

                insert(value,0,i_y,buffer);

                value = val(limit_x  ,i_y  ,*this_) * 0.5f   +
                        val(limit_x  ,i_y+1,*this_) * 0.125f +
                        val(limit_x  ,i_y-1,*this_) * 0.125f +
                        val(limit_x-1,i_y  ,*this_) * 0.125f;

                insert(value,limit_x,i_y,buffer);
        }

        /* lastly - corners */

        value = val(0,0,*this_) * 0.5f   +
                val(1,0,*this_) * 0.125f +
                val(0,1,*this_) * 0.125f;

        insert(value,0,0,buffer);

        value = val(limit_x  ,0,*this_) * 0.5f   +
                val(limit_x-1,0,*this_) * 0.125f +
                val(limit_x  ,1,*this_) * 0.125f;

        insert(value,limit_x,0,buffer);

        value = val(0,limit_y  ,*this_) * 0.5f   +
                val(1,limit_y-1,*this_) * 0.125f +
                val(0,limit_y  ,*this_) * 0.125f;

        insert(value,0,limit_y,buffer);

        value = val(limit_x  ,limit_y  ,*this_) * 0.5f   +
                val(limit_x-1,limit_y  ,*this_) * 0.125f + 
                val(limit_x  ,limit_y-1,*this_) * 0.125f;

        insert(value,limit_x,limit_y,buffer);


/* end of edges */

        tmp = *this_;
     *this_ = buffer;
     buffer = tmp;
     
    }

    destroyArray(&buffer);
}

/*
*
* run the algorithm with predefined values
*
*/


void go(ulong iterations,gridArrayPtr* this_) {

    ulong iter = 0;
    gridArrayPtr buffer = NULL;
    float snipVal = 2.0f;

    for (iter = 0;iter < iterations;++iter) {

#ifdef TE
        std::cout << "#";
#endif

	      snipArray(*this_,
                  (ulong)(size_x(*this_)/snipVal),
                  (ulong)(size_y(*this_)/snipVal),
                  &buffer);

        snipVal *= 2.0f;

        scale(size_x(*this_),size_y(*this_),&buffer);

        mulScalar(2.0f,buffer);
        
        addArray(*this_,buffer);
        
        filter(3,this_);
        
        destroyArray(&buffer);
    }
}

/*
*
* Normalisation to a range of 0..hi
*
*/

void normalize(float hi,gridArrayPtr this_) {

//    1. find largest value
    float largest = -4000000000.0f;
    float smallest = 4000000000.0f;
    ulong limit_x = size_x(this_);
    ulong limit_y = size_y(this_);
    ulong i_x = 0;
    ulong i_y = 0;

//// shift the map to range 0..somevalue
//for (i_x = 0;i_x < limit_x;++i_x)
//{
//    for (i_y = 0;i_y < limit_y;++i_y)
//    {
//        if (val(i_x,i_y,this_) < smallest)
//        {
//            smallest = val(i_x,i_y,this_);
//        }
//    }
//}
//
////add smallest
//
//if (smallest < 0.0f)
//    addScalar(smallest * -1.0,this_);


for (i_x = 0;i_x < limit_x;++i_x)
{
    for (i_y = 0;i_y < limit_y;++i_y)
    {
        if (val(i_x,i_y,this_) > largest)
        {
            largest = val(i_x,i_y,this_);
        }
    }
}

for (i_x = 0;i_x < limit_x;++i_x)
{
    for (i_y = 0;i_y < limit_y;++i_y)
    {
        normalizePoint(i_x,i_y,0.0f,largest,hi,this_);
    }
}

}

/*
*
* Constrain adjacent vertices to differ no more than +1/-1 in height
* (inline to allow compiler to optimize the comparison type)
*
*/

static inline void constrain(int raise, ulong range, gridArrayPtr this_) {
  int limit_x = size_x(this_);
  int limit_y = size_y(this_);

  int i_x;
  int i_y;
  
#ifndef TE 

  ulong l_ind = 1;
  ulong l_lim = 2;

  
/* pass the map in four directions to set it so the
   map is in the TTD format (no more then 1 height difference
   in each direction */

  for (i_x = 0; i_x < limit_x; ++i_x)
      for (i_y = l_ind; i_y < limit_y-l_lim; ++i_y)
          adjust(i_x, i_y, i_x, i_y-1, raise, range, this_);
  
  for (i_y = limit_y-1; i_y > 0; --i_y)
      for (i_x =  l_ind ;i_x < limit_x-l_lim; ++i_x)
          adjust(i_x, i_y, i_x-1, i_y, raise, range, this_);

  for (i_x = limit_x-1;i_x > 0;--i_x)
      for (i_y = limit_y-l_lim;i_y > l_ind;--i_y)
          adjust(i_x, i_y, i_x, i_y+1, raise, range, this_);

  for (i_y = 0;i_y < limit_y;++i_y)
      for (i_x = limit_x-l_lim;i_x > l_ind;--i_x)
          adjust(i_x, i_y, i_x+1, i_y, raise, range, this_);

#else
  
/* pass the map in four directions to set it so the
   map is in the TE format (no more then 7 height difference
   in each direction
   note that unlike ttd, here the edges do not have the req
   uirement for 0 at edge!*/

  for (i_x = 0; i_x < limit_x; ++i_x)
      for (i_y = 0; i_y < limit_y; ++i_y)
          adjust(i_x, i_y, i_x, i_y-1, raise, range, this_);
  
  for (i_y = limit_y-1; i_y >= 0; --i_y)
      for (i_x =  0 ;i_x < limit_x; ++i_x)
          adjust(i_x, i_y, i_x-1, i_y, raise, range, this_);

  for (i_x = limit_x-1;i_x >= 0;--i_x)
      for (i_y = limit_y-1;i_y >= 0;--i_y)
          adjust(i_x, i_y, i_x, i_y+1, raise, range, this_);

  for (i_y = 0;i_y < limit_y;++i_y)
      for (i_x = limit_x-1;i_x >= 0;--i_x)
          adjust(i_x, i_y, i_x+1, i_y, raise, range, this_);


#endif
}

/*
*
* Transform into TTD format
*
*/

void constrainMap(ulong cutDown, ulong cutUp, ulong tileRange, gridArrayPtr this_) {

/* set the limits to avoid calling size_x() and size_y()
   methods on each loop */



    ulong limit_x = size_x(this_);
    ulong limit_y = size_y(this_);
    ulong i_x = 0;
    ulong i_y = 0; 

/* normalize to the proper range and subtract the cutDown */

#ifndef TE
    float val1 = 1.0f*cutUp+13.0f+cutDown*1.0f/4.0f;
#else
    cutUp *= tileRange;
    cutDown *= tileRange;
    float val1 = 1.0f*cutUp+13.0f*tileRange+cutDown*1.0f/4.0f;
#endif

    normalize(val1,this_);
    addScalar((cutUp* - 1.0f),this_);
    normalize(float(cutDown),this_);

/* first pass to adjust adjacent heights by raising neighbors as needed */


/* zero two outmost vertices
   not needed for TE, so commented out */


#ifndef TE

    constrain(1, tileRange, this_);


    for (i_x = 0;i_x < limit_x;++i_x) {
        insert(0.0f,i_x,        0,this_);
        insert(0.0f,i_x,        1,this_);
        insert(0.0f,i_x,limit_y-2,this_);
        insert(0.0f,i_x,limit_y-1,this_);
    }

    for (i_y = 0;i_y < limit_y;++i_y) {
        insert(0.0f,        0,i_y,this_);
        insert(0.0f,        1,i_y,this_);
        insert(0.0f,limit_x-2,i_y,this_);
        insert(0.0f,limit_x-1,i_y,this_);
    }

    constrain(0, tileRange, this_);

/* constrain neighbours again, this time lowering as needed so as not
   to raise map edges
   */

    for (i_x = 0; i_x < limit_x;++i_x)
    {
        for (i_y = 0; i_y < limit_y;++i_y)
        {
            if (val(i_x,i_y,this_) > 15.0f) insert (15.0f, i_x, i_y, this_);
            if (val(i_x,i_y,this_) <  0.0f) insert (0.0f, i_x, i_y, this_);
        }
    }
#else
    constrain(1, tileRange, this_);
 //   constrain(0, tileRange, this_);


    for (i_x = 0; i_x < limit_x;++i_x)
    {
        for (i_y = 0; i_y < limit_y;++i_y)
        {
            if (val(i_x,i_y,this_) <  0.0f) insert (0.0f, i_x, i_y, this_);
            if (val(i_x,i_y,this_) >  4095.0f) insert (4095.0f, i_x, i_y, this_);
        }
    }

    constrain(0, tileRange, this_);

#endif
}

/*
*
* Create a desert where applicable
*
*/

void ttDesert(gridArrayPtr* this_) {

     cellularAutomata(randomfn()%5+3,randomfn()%5+2, 65, this_);
     
     filter(3,this_);
     scale(128,128,this_);
     filter(3,this_);
     scale(256,256,this_);
     filter(3,this_);    
}

/*
*
* test recursively if a tile is sensibly located
*
*/

void shoreStomp(ulong i_x, ulong i_y,gridArrayPtr geometry, gridArrayPtr this_)
{
    int j_x = 0;
    int j_y = 0;
    
    int c_x = 0;
    int c_y = 0;
    
    static float tab [7][7] = {
                         { 1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 1.0 },
                         { 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0 },
                         { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                         { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                         { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                         { 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0 },
                         { 1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 1.0 }
                       };
                       
    if (val(i_x,i_y,geometry) < 1.05f)
    {
      for (j_x = 0;j_x < 7;++j_x)
        {
        for (j_y = 0;j_y < 7;++j_y)
         {
           c_x = i_x + j_x - (randomfn()%2 + 3);
           c_y = i_y + j_y - (randomfn()%2 + 3);
           insert(val(c_x,c_y,this_)*tab[j_x][j_y], c_x, c_y, this_);
         } 
        }
      }
}

/*
*
* destroy array
*
*/

void destroyArray(gridArrayPtr* this_) {

  ulong i_x;

  for (i_x = 0;i_x < size_x(*this_);++i_x)
    free((*this_)->data[i_x]);

  free((*this_)->data);
#ifdef TE
  delete *this_;
#else if
  free(*this_);
#endif
  *this_ = NULL;
}


/*
*
* multiply elements of this_ by elements of source_
* at position pos
*
* it is NOT matrix multiplication!!!
*
*/

void mulArrayStrip(ulong pos, gridArrayPtr source_, gridArrayPtr this_)
{
    ulong limit_y = size_y(this_);
    ulong i_y = 0;

    if (pos >= limit_y) return;

    for (;i_y < limit_y;++i_y)
    {
        this_->data[pos][i_y] *= val(pos,i_y,source_);
    }
}

/* multiply elements of one array with elements of another array */

void mulArray(gridArrayPtr source_,gridArrayPtr this_) {

    ulong limit_x = 0;
    ulong i_x = 0;

    // check if the array sizes match
    if (size_x(this_) != size_x(source_) && size_y(this_) != size_y(source_))
    {
        return;
    }
    // do the multiplication

    
    limit_x = size_x(this_);
#ifdef TE

    /* thSpeci1 is used to indicate number of threads,
       thSpeci2 is used to indicate the next protected
       section */
    while (this_->thSpeci2 < limit_x)
    {
        int my_val;
        {
            boost::mutex::scoped_lock lk (this_->my_mutex_);
            my_val = this_->thSpeci2;
            this_->thSpeci2 += 4;
        }
        this_->thSpeci1++;
        mulArrayStrip(my_val,source_,this_);
        mulArrayStrip(my_val+1,source_,this_);
        mulArrayStrip(my_val+2,source_,this_);
        mulArrayStrip(my_val+3,source_,this_);
        this_->thSpeci1--;
    }

    if (this_->thSpeci1 == 0) // the last thread to exit
    {
        this_->thSpeci2 = 0;
    }

#else if
 
    for (i_x = 0;i_x < limit_x;++i_x)
    {
        mulArrayStrip(i_x,source_,this_);
    }
#endif

}

/* move a given rown one place */

void shift_x (int dir, ulong row, gridArrayPtr this_) {
     
     ulong i_x = 0;
     float tmp = 0;
     
     if (this_->size_x <= row) return;
     
     if (dir > 0)
     {
       tmp = this_->data[0][row];
       for (i_x = 1;i_x < this_->size_x;++i_x)
       {
          this_->data[i_x-1][row] = this_->data[i_x][row];
       }
       this_->data[this_->size_x-1][row] = tmp;
     }
     
     if (dir < 0)
     {
       tmp = this_->data[this_->size_x-1][row];
       for (i_x = this_->size_x-2;i_x > -1;--i_x)
       {
          this_->data[i_x+1][row] = this_->data[i_x][row];
       }
       this_->data[0][row] = tmp;
     }
     
}

/*
*/

void shift_y (int dir, ulong row, gridArrayPtr this_) {
     
     ulong i_y = 0;
     float tmp = 0;
     
     if (this_->size_y <= row) return;
     
     if (dir > 0)
     {
       tmp = this_->data[row][0];
       for (i_y = 1;i_y < this_->size_y;++i_y)
       {
          this_->data[row][i_y-1] = this_->data[row][i_y];
       }
       this_->data[row][this_->size_y-1] = tmp;
     }
     
     if (dir < 0)
     {
       tmp = this_->data[row][this_->size_y-1];
       for (i_y = this_->size_y-2;i_y > -1;--i_y)
       {
          this_->data[row][i_y+1] = this_->data[row][i_y];
       }
       this_->data[row][0] = tmp;
     }
}

/* cellular automata */

void infest(gridArrayPtr source, ulong i_x, ulong i_y, gridArrayPtr this_)
{
    if (val (i_x, i_y, source) != 0.0f)
        return;

    int neiCount = 0;
    float neiVal = 0.0f;

    neiVal = val(i_x+1,i_y,source);
    if (neiVal == -1.0f)
       neiCount += 1;
    if (neiVal == 1.0f)
       neiCount -= 1;
    
    neiVal = val(i_x-1,i_y,source);
    if (neiVal == -1.0f)
       neiCount += 1;
    if (neiVal == 1.0f)
       neiCount -= 1;
    
    neiVal = val(i_x,i_y+1,source);
    if (neiVal == -1.0f)
       neiCount += 1;
    if (neiVal == 1.0f)
        neiCount -= 1;
    
    neiVal = val(i_x,i_y-1,source);
    if (neiVal == -1.0f)
       neiCount += 1;
    if (neiVal == 1.0f)
       neiCount -= 1;

    neiCount *= 25;

    if (neiCount > 0)
       {
           if (int(randomfn()%100) < neiCount)
           insert(-1.0f,i_x,i_y,this_);
           else
           neiCount = 0;
       }
 
    if (neiCount < 0)
       {
           if (int(randomfn()%100) < -neiCount)
           insert(1.0f,i_x,i_y,this_);
           else
           neiCount = 0;
       }

    if (neiCount == 0)
       {
           insert(0.0f,i_x,i_y,this_);
       }
}

#define SOURCE_SIZE 64

void cellularAutomata(ulong pop1, ulong pop2, ulong iters, gridArrayPtr* this_)
{

     
     ulong posX = 0;
     ulong posY = 0;
     ulong limit_x = SOURCE_SIZE;
     ulong limit_y = SOURCE_SIZE;
     ulong i_x = 0;
     ulong i_y = 0;
     ulong changed = 0;
     mulScalar(0.0,*this_);
     gridArrayPtr tmp = NULL;
     snipArray(*this_,limit_x,limit_y,&tmp);
     
     for (changed = 0; changed < pop1;++changed) {
       posX = randomfn()%SOURCE_SIZE;
       posY = randomfn()%SOURCE_SIZE;
       insert(-1.0,posX,posY,*this_);
     }
     
     for (changed = 0; changed < pop1;++changed) {
       posX = randomfn()%SOURCE_SIZE;
       posY = randomfn()%SOURCE_SIZE;
       insert(1.0,posX,posY,*this_);
     }
     
     changed = 0;
     
     while (changed < iters)
     {
     changed++;

     for (i_x = 0;i_x < limit_x;++i_x)
         {
         for (i_y = 0;i_y < limit_y;++i_y)
             {
               infest(*this_,i_x,i_y,tmp);
             }
         }
     destroyArray(this_);
     *this_ = NULL;
     snipArray(tmp,limit_x,limit_y,this_);
     }
 }
