#ifndef __COMMUNICATION_H__
#define __COMMUNICATION_H__

/*
* @brief this class defines the interface used by the client and
*        server to send data to each other
*/

#include "action_map.h"
#include "map.h"
#include "boost/thread/mutex.hpp"
#include <list>
#include <vector>

namespace te
{
//#############################################################################
/*
* @brief datagram carries the data sent between client and server
*
*/
class datagram
{
public: //---------------------------------------------------------------------
    datagram()
    {
        command_ = dummy;      // carried command
    }
//-----------------------------------------------------------------------------
    int command_;
    std::vector<size_t> params_; // carried command parameters
};
//#############################################################################
class commLink
{
public: //---------------------------------------------------------------------
    commLink()
    {
        my_payload_.resize(0);
    }
//-----------------------------------------------------------------------------
    virtual ~commLink()
    {}
//-----------------------------------------------------------------------------
    /*
    * @brief fetch a single datagram from communication
    *
    * @param a place to store the data
    * @return return true if the dgram was read, or false otherwise
    */
    virtual bool fetch(datagram& dgram)
    {
        if (my_payload_.size() > 0)
        {
            dgram = *(my_payload_.begin());
            my_payload_.erase(my_payload_.begin());
            return true;
        }
        my_payload_.clear();
        return false;
    };
//-----------------------------------------------------------------------------
    /*
    * @brief put a single datagram onto the datagram list
    *
    * @param a datagram to be dispatched
    * @return true if success, false otherwise
    */
    virtual bool dispatch(datagram& dgram)
    {
        boost::mutex::scoped_lock lk(my_mutex_);
        my_payload_.push_back(dgram);
        return false;
    };
protected: //------------------------------------------------------------------
    std::list<datagram> my_payload_;
    boost::mutex  my_mutex_;
};
////#############################################################################
//class  serverCommLink : public commLink
//{
//public: //---------------------------------------------------------------------
//serverCommLink(terrain_ptr& its_terrain) : my_terrain_(its_terrain)
//{
//    my_payload_.resize(0);
//}
////-----------------------------------------------------------------------------
//virtual bool fetch(datagram& dgram)
//{
//begin:
//    if (my_payload_.size() > 0)
//    {
//        dgram = *(my_payload_.begin());
//        switch (dgram.command_)
//        {
//        case area_req:  processAreaRequest(dgram);
//                        my_payload_.erase(my_payload_.begin());
//                        goto begin;                     
//        }
//        my_payload_.erase(my_payload_.begin());
//        return true;
//    }
//    my_payload_.clear();
//    return false;
//}
////-----------------------------------------------------------------------------
//void processAreaRequest(const datagram& dgram)
//{
//
//}
//protected: //------------------------------------------------------------------
//terrain_ptr& my_terrain_;
//};
//#############################################################################
};


#endif