#ifndef __EVENT_H__
#define __EVENT_H__

#include <iostream>
#include <OIS.h>
#include <OISMouse.h>
#include <OISKeyboard.h>
#include <OISInputManager.h>
#include "renderer.h"
#include "boost/lexical_cast.hpp"
#include "action_map.h"

// event handling enviroment

namespace te
{
//#############################################################################
class eventReader2 : OIS::MouseListener, OIS::KeyListener
{
public: //---------------------------------------------------------------------

    eventReader2(action_map& its_actions,
                   renderer& its_renderer,
                      size_t wHandle,
                      size_t wWidth,
                      size_t wHeight)

        : my_actions_ (its_actions),
         my_renderer_ (its_renderer)
    {
        wHandle_ = wHandle;

        keyboard_ = 0;
        mouse_ = 0;
        inputManager_ = 0;

        OIS::ParamList  paramList;
        paramList.insert(  std::make_pair                (          \
                           std::string                   ("WINDOW"),\
                         boost::lexical_cast<std::string>(static_cast<unsigned int>(wHandle_))\
                                                         )\
                        );
        
        inputManager_ = OIS::InputManager::createInputSystem (paramList);
        inputManagerInstances_ = 1;

        if (inputManager_->numKeyBoards() > 0)
        {
            keyboard_ = static_cast<OIS::Keyboard*>\
                (inputManager_->createInputObject(OIS::OISKeyboard,true));
            keyboard_->setEventCallback(this);
            keyboardInstances_  = 1;
        }

        if (inputManager_->numMice() >  0) 
        {
            mouse_ = static_cast<OIS::Mouse*>\
                (inputManager_->createInputObject(OIS::OISMouse,true));
            mouse_->setEventCallback(this);
            mouseInstances_ = 1;
            
            const OIS::MouseState& mouseState = mouse_->getMouseState();
            mouseState.width  = wWidth;
            mouseState.height = wHeight;
        }
    }

//-----------------------------------------------------------------------------

    eventReader2 (eventReader2& src) :
       
        my_actions_(src.my_actions_),
       my_renderer_(src.my_renderer_)
    {
        this->inputManager_ = src.inputManager_;
        if (this->inputManager_)
            inputManagerInstances_++;

        this->keyboard_ = src.keyboard_;
        if (this->keyboard_)
            keyboardInstances_++;

        this->mouse_ = src.mouse_;
        if(this->mouse_)
            mouseInstances_++;

        this->wHandle_ = src.wHandle_;
    }

//-----------------------------------------------------------------------------

    ~eventReader2()
    {

        if ( mouseInstances_ == 1 ) {
            this->inputManager_->destroyInputObject ( mouse_ );
            mouse_ = 0;
            mouseInstances_--;
        }

        if( keyboardInstances_  == 1 ) {
            this->inputManager_->destroyInputObject ( keyboard_ );
            keyboard_ = 0;
            keyboardInstances_--;
        }

        if ( inputManagerInstances_ == 1 )
        {
            this->inputManager_->destroyInputSystem( this->inputManager_ );
            this->inputManager_ = 0;
            inputManagerInstances_--;
        }

    }

//-----------------------------------------------------------------------------

 void operator()()
    {
        if (    mouse_ )    mouse_->capture();
        if ( keyboard_ ) keyboard_->capture(); 
    }

//-----------------------------------------------------------------------------

//-  define to ease the downcasting
#define AMM_PTR \
    static_cast<action_map_make*>(my_actions_[map_make].get())
//-

bool keyPressed (const OIS::KeyEvent &event)
    {
        switch (event.key)
        {
        case OIS::KC_ESCAPE : my_actions_[exit    ]->invoke(); break;
        case OIS::KC_M      : AMM_PTR->setSize(256,256);
                              my_actions_[map_make]->invoke();
                              break;
        case OIS::KC_EQUALS : my_renderer_.cameraFovDec(true); break;
        case OIS::KC_MINUS  : my_renderer_.cameraFovInc(true); break;
        
        }
        std::cout << "key press";
        return true;
    }

//- undef the previous to not pollute global namespace
#undef AMM_PTR
//-

//-----------------------------------------------------------------------------

bool keyReleased (const OIS::KeyEvent &event)
    {
        switch (event.key)
        {
        case OIS::KC_ESCAPE: break;
        case OIS::KC_M : break;
        case OIS::KC_EQUALS : my_renderer_.cameraFovDec(false); break;
        case OIS::KC_MINUS  : my_renderer_.cameraFovInc(false); break;

        default: break;
        }
        std::cout << "key depress";
        return true;
    }

//-----------------------------------------------------------------------------

bool mousePressed( const  OIS::MouseEvent& event, OIS::MouseButtonID id)
    {
        std::cout << "mouse key press";
        return true;
    }

//-----------------------------------------------------------------------------

bool mouseReleased( const  OIS::MouseEvent& event, OIS::MouseButtonID id)
    {
        std::cout << "mouse key release";
        return true;
    }

//-----------------------------------------------------------------------------

bool mouseMoved( const  OIS::MouseEvent& event)
    {
        std::cout << "mousemove";
        return true;
    }

//-----------------------------------------------------------------------------


private:

    size_t              wHandle_;
    OIS::Mouse*         mouse_;
    size_t              mouseInstances_;
    OIS::Keyboard*      keyboard_;
    size_t              keyboardInstances_;
    OIS::InputManager*  inputManager_;
    size_t              inputManagerInstances_;
    action_map&         my_actions_;
    renderer&           my_renderer_;

//-----------------------------------------------------------------------------

};

};

#endif
