#include "exco.h"
using namespace ec;

//-----------------------------------------------------------------------------
task::task(exco& its_list) : my_list_(its_list)
{
};
//-----------------------------------------------------------------------------
task::~task()
{
};
//#############################################################################
exco::excoIterator::excoIterator()
{
    isInitialized_ = false;
}
//-----------------------------------------------------------------------------
void exco::excoIterator::operator++()
{
    ++iter;
}
//#############################################################################
exco::exco(size_t numOfUsers) :\
    service_barr_    (static_cast<unsigned int>(numOfUsers)),
    numOfUsers_      (numOfUsers)
{
    entryCount_ = 0;
    isInitialized_ = false;
    finishedCount_ = 0;
}
//-----------------------------------------------------------------------------
exco::~exco()
{
}
//-----------------------------------------------------------------------------
void exco::insert(const task_ptr& task)
{
    boost::mutex::scoped_lock lk(insert_mtx_);
    tail.push_back(task);
}
//-----------------------------------------------------------------------------
void exco::flush()
{
    boost::mutex::scoped_lock lk(insert_mtx_);
    
    excoListIterator indice = tasks.begin();
    excoListIterator bumper = tasks.end();
    excoListIterator previous = indice;

    //std::cout << " " << tasks.size() << " ";

    while(indice != bumper)
    {
        if (indice == previous && (*indice)->isObsolete())
        {
            tasks.erase(indice);
            indice = tasks.begin();
            previous = indice;
        } else {
            if ((*indice)->isObsolete())
            {
                tasks.erase(indice);
                indice = previous;
                ++indice;
            } else {
                previous = indice;
                ++indice;
            }
        }
    }
    //std::cout << this->tasks.size() << " ";
}
//-----------------------------------------------------------------------------
void  exco::clear()
{
    tasks.clear();
}
//-----------------------------------------------------------------------------
bool exco::execute(excoIterator& iter)
{
    if (!isInitialized_)
    {
        if(service_barr_.wait())
        {
            merge();
            isInitialized_ = true;
            entryCount_ = 0;
        }
        service_barr_.wait();
    }

    if (iter.isInitialized_)
    {
        //--------------------------------------------
        if (iter.iter == tasks.end())
        {
            service_barr_.wait();
            //std::cout << unsigned int(finishedCount_) << " " << unsigned int(tasks.size()) << "\n";
            if (finishedCount_ >= tasks.size())
            {
                if(service_barr_.wait())
                {
                    finishedCount_ = 0;
                    merge();
                }
                iter.isInitialized_ = false;
                return false;
            } else {
                iter.iter = tasks.begin();
                return true;
            }
            service_barr_.wait();
        }
        //--------------------------------------------
        if (iter.iter != this->tasks.end())
        {
            (*(iter.iter))->invoke(finishedCount_);
            ++iter;
        }
        return true;
        //--------------------------------------------
    }
    else
    {
        merge();
        iter.iter = this->tasks.begin();
        iter.isInitialized_ = true;
        service_barr_.wait();
        return true;
    }
}
//-----------------------------------------------------------------------------
bool exco::refresh()
{
    excoListIterator indice = tasks.begin();
    excoListIterator bumper = tasks.end();
    for (;indice!=bumper;++indice)
    {
        (*indice)->refresh(false);
    }
    this->finishedCount_ = 0;
    return true;
}
//-----------------------------------------------------------------------------
void exco::merge()
{
    tasks.merge(tail);
    tail.clear();
}
//-----------------------------------------------------------------------------
bool exco::isCompleted()
{
    excoListIterator indice = tasks.begin();
    excoListIterator bumper = tasks.end();
    for (;indice!=bumper;++indice)
    {
        if (!(*indice)->isFinished())
            return false;
    }
    return true;
}
//#############################################################################
executor::executor(exco& target, std::string id) : my_list_(target)
{
    id_ =  id;
    my_delegate_ = new delegate(*this);
    thr_ = boost::shared_ptr<boost::thread>(new boost::thread(*my_delegate_));
    isFinished_ = false;
}
//-----------------------------------------------------------------------------
executor::~executor()
{
    delete my_delegate_;
}
//-----------------------------------------------------------------------------
bool executor::isFinished()
{
    return isFinished_;
}
//#############################################################################
executor::delegate::delegate(executor& parent) : parent_(parent)
{
};
//-----------------------------------------------------------------------------
executor::delegate::delegate(const delegate& src) : parent_(src.parent_)
{
};
//-----------------------------------------------------------------------------
int executor::delegate::operator()()
{
    parent_.my_delegate_ = this;
    isFinished_ = false;
    while(1)
    {
        if (!parent_.my_list_.execute(parent_.my_iterator_))
        {
            //std::cout << ("end\n");
            parent_.isFinished_ = true;
            isFinished_ = true;
        }
    }
    return 0;
}
//#############################################################################
threadlessExecutor::threadlessExecutor(exco& target, std::string id) : my_list_(target)
{
    id_ =  id;
}
//-----------------------------------------------------------------------------
threadlessExecutor::~threadlessExecutor()
{
}
//-----------------------------------------------------------------------------
int threadlessExecutor::go()
{
    while(1)
    {
        if (!my_list_.execute(my_iterator_))
        {
            //std::cout << ("end\n");
            break;
        }
    }
    return 0;
}
//#############################################################################
