/* 
main.cpp: main loop, option handling

Copyright 2006, 2007 Adam Kadlubek
Copyright 2007 Nathanael Nerode

This file is part of TRANSPORT EMPIRE (TE).

TE is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

TE is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with TE; see the file COPYING.  If not, write to the Free
Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.  
*/

#include "exco.h"
#include "communication.h"
#include "server.h"
#include "client.h"
#include "event.h"
#include "renderer.h"
#include "map.h"
#include "common/gridArray.h"
#include "random.h"

#ifdef WIN32
  #include "timing.h"
#else /* WIN32 */
  #include "timing_posix.h"
#endif

using namespace te;

std::string pluginsName;

int main(int argc, char** argv)
{
    int numOfUsers = 1;
    pluginsName = "plugins.cfg";
    bool framelimit = true;
    std::string arg;

    int argIndex = 1;
    for (argIndex = 1; argIndex + 1 < argc; argIndex += 2)
    {
	/* We always have two arguments left, the current one and the next one */
        arg = argv[argIndex];
        if (arg == "-threads")
            numOfUsers = boost::lexical_cast<int>(argv[argIndex+1]);

        if (arg == "-plugins")
            pluginsName = argv[argIndex+1];

        if (arg == "-nolimit")
        {
            arg = argv[argIndex+1];
            if (arg == "yes")
                framelimit = false;
            else
                framelimit = true;
        }
    }

    if (numOfUsers < 1) numOfUsers = 1;

    ec::exco ex(numOfUsers);

    te::terrain_ptr ter;
    te::terrain_ptr ter2;

    te::commLink comm;
    te::commLink comm2;
    
    ec::task_ptr sptr (new te::server(ex,comm,comm2,numOfUsers,ter));
    ec::task_ptr cptr (new te::client(ex,comm2,comm,numOfUsers,ter2));
    
    ex.insert(sptr);
    ex.insert(cptr);

    ec::threadlessExecutor exc1(ex, "main");

    typedef boost::shared_ptr<ec::executor> exc_ptr;

    std::list<exc_ptr> executors;
    executors.resize(0);

    for (int x = 1;x < numOfUsers;++x)
    {
        executors.push_back(exc_ptr(new ec::executor(ex,"executor")));
    }
    
    int ret = 0;
    te::runTime rn;
    float frameTime = 0;
    int frame = 0;

    while(!(*(static_cast<te::server*>(sptr.get()))).isExited())
    {
        rn.now();
        if (frame == 96)
        {
            frame = 0;
            std::cout << "Average frame time:" << frameTime/96 << "\n\n";
            frameTime = 0;
        } 
        exc1.go();
        ex.refresh();
        frameTime += rn.later();
        //std::cout <<">"<< rn.later();
        if (framelimit)
            if (rn.later() <= 31)
            {
                te::pause(31 - rn.now());
            }
        frame++;
    }
    return 0;
}
