/*
This file is a part of TRANSPORT EMPIRE
2006-2007 Copyright by Adam Kadlubek.

This file is licenced under GPL General Public Licence
*/

#ifndef __RENDER_TERRAIN_DISPLAY_LIST__
#define __RENDER_TERRAIN_DISPLAY_LIST__

#include "../mutexes.h"
#include "render_terrain_node.h"
#include <list>

namespace te
{
//#############################################################################
    /**
    * @remark
    *    this class is used to implement a list of terrain display nodes
    *    it is used to cache and control display of terrain leafs
    */
class render_terrain_display_list
{
public: //---------------------------------------------------------------------
    //#########################################################################
        /**
        * @remark
        *    used to hold an element of a list in a specified way
        */
    class extractor
    {
        friend class render_terrain_display_list;
    public: //-----------------------------------------------------------------
        /**
        * @remark
        *   obtain a reference to a list element
        */
    render_terrain_node& operator()()
    {
        return (*iter);
    }
    private: //----------------------------------------------------------------
        std::list<render_terrain_node>::iterator iter;
    };
    //#########################################################################
    /**
    * @remark
    *   constructor
    */
render_terrain_display_list(size_t size, renderer& node_renderer_)
{
    for (size_t x = 0; x < size;++x)
    {
        my_list_.push_back(render_terrain_node(node_renderer_));
    }
};
//-----------------------------------------------------------------------------
    /**
    * @remark
    *   destructor
    */
~render_terrain_display_list()
{
    std::cout << "List destroyed GAME OVAR!";
};
//-----------------------------------------------------------------------------
    /**
    * @remark
    *   obtain a reference to the last element of the list (guaranteed that
    *   it is not displayed
    * @param
    *   extractor used to reference the extracted element
    */
void getLast(extractor& iter)
{
    lock lk(list_operation_);
    int x = (*(--my_list_.end())).mynum;
    iter.iter = (--my_list_.end());
};
//-----------------------------------------------------------------------------
    /**
    * @remark
    *   move the element held by an extractor to the front of the list
    *   used to signify an element that needs to be redrawn (only first x
    *   elements will be drawn)
    * @param
    *   extractor holding a reference to an element which is to be moved to front
    */
void moveToFront(extractor& iter)
{
    lock lk(list_operation_);
    my_list_.insert(my_list_.begin(),*iter.iter);
    my_list_.erase(iter.iter);
    iter.iter = my_list_.begin();
};
//-----------------------------------------------------------------------------
    /**
    * @remark
    *   destroy geometry of the whole list - done on map resize ferinstance
    */
    void flush()
    {
        std::list<render_terrain_node>::iterator indice = my_list_.begin();
        std::list<render_terrain_node>::iterator bumper = my_list_.end();
        for (;indice != bumper;++indice)
        {
            indice->destroyGeometry();
        }
    }
private: //--------------------------------------------------------------------
    std::list<render_terrain_node> my_list_;
    mutex                          list_operation_;
};
//#############################################################################
};

#endif
