
/*
This file is a part of TRANSPORT EMPIRE
2006-2007 Copyright by Adam Kadlubek.

This file is licenced under GPL General Public Licence
*/

#ifndef __RENDER_TERRAIN_HOOK__
#define __RENDER_TERRAIN_HOOK__

#include "render_terrain_node.h"
#include "render_terrain_display_list.h"
#include "boost/shared_ptr.hpp"
#include "../renderer.h"

namespace te
{
/**
    @remark
        this class binds the map leaf with renderer scene node
        it controls if it is bound to a leaf_drawer and if the
        leaf in question needs to be redrawn

        each leaf on the map has one hook.
    */
class render_terrain_hook
{
public: //-----------------------------------------------------------------
    render_terrain_hook(leaf<tileData>& its_leaf,\
                              renderer& its_renderer,\
           render_terrain_display_list& its_display_list,
                                 size_t its_longitude,
                                 size_t its_lattitude,
                                size_t& its_dayOfYear,
                                 size_t its_map_longitunal_size_) :

                               my_leaf_(its_leaf),\
                           my_renderer_(its_renderer),\
                       my_display_list_(its_display_list),\
                          my_lattitude_(its_lattitude),\
                          my_longitude_(its_longitude),\
                 last_rendered_version_(0),\
                            my_counter_(0),\
                          my_dayOfYear_(its_dayOfYear)
    {
        my_display_list_.getLast(my_node_);
        my_node_().my_hook_ = this;
        my_drawing_index_ = (my_longitude_*its_map_longitunal_size_ + my_lattitude_)%128*8;
    };
//-------------------------------------------------------------------------
    ~render_terrain_hook()
    {
    };
//-------------------------------------------------------------------------
    /**
    * @remark
    *   prepare the leaf the hook holods to be displayed - that is attach
    *   all tiles to it
    */
void build()
{
    // check if the node we are using is pointing at us

    render_terrain_hook* me = this;

    if (my_node_().my_hook_ != me)
    {
        // our node has been taken from us
        // take a free one and build the frame
        my_display_list_.getLast(my_node_);
        my_display_list_.moveToFront(my_node_);
        my_node_().my_hook_ = this;
        my_renderer_.buildTerrainLeaf(my_leaf_,my_node_().getShadow(),\
            my_longitude_, my_lattitude_,my_node_().getShadowChildren());
        last_rendered_version_ = my_leaf_.getVersion();
        my_node_().makeRenderable();
        my_node_().insertIntoScene();
        return;
    }

    // determine if the terrain needs to be re-rendered
    // due to version change

    if (my_leaf_.getVersion() > last_rendered_version_)
    {
        // we have a node, build the leaf
        my_renderer_.buildTerrainLeaf(my_leaf_,my_node_().getShadow(),\
            my_longitude_, my_lattitude_,my_node_().getShadowChildren());
        my_node_().makeRenderable();
        my_display_list_.moveToFront(my_node_);
        my_node_().insertIntoScene();
        last_rendered_version_ = my_leaf_.getVersion();
        return;
    }

    //force redraw each 5 days
    
    my_interval_ = my_dayOfYear_ >> 3;
    ++my_counter_;

    if (my_last_interval_ != my_interval_)
    {
        my_last_interval_ = my_interval_;
        my_counter_ = 0;
    }

    if (my_counter_ == my_drawing_index_)
    {
        my_renderer_.buildTerrainLeaf(my_leaf_,my_node_().getShadow(),\
            my_longitude_, my_lattitude_,my_node_().getShadowChildren());
        my_node_().makeRenderable();
        my_display_list_.moveToFront(my_node_);
        my_node_().insertIntoScene();
        last_rendered_version_ = my_leaf_.getVersion();
        //std::cout << "!" << my_longitude_ << " " << my_lattitude_ << "!";
        return;
    }

    my_node_().makeNonRenderable();
}
private: //----------------------------------------------------------------
                                     size_t last_rendered_version_;
                                     size_t current_version_;
                            leaf<tileData>& my_leaf_;
                                  renderer& my_renderer_;
               render_terrain_display_list& my_display_list_;
     render_terrain_display_list::extractor my_node_;
                                     size_t my_longitude_;
                                     size_t my_lattitude_;
                                     size_t my_interval_;
                                     size_t my_last_interval_;
                                    size_t& my_dayOfYear_;
                                     size_t my_drawing_index_;
                                     size_t my_counter_;
};
//#########################################################################
typedef boost::shared_ptr<render_terrain_hook> hook_ptr;
//#########################################################################


};
#endif
