/*
This file is a part of TRANSPORT EMPIRE
2006-2007 Copyright by Adam Kadlubek.

This file is licenced under GPL General Public Licence
*/

#ifndef __RENDER_TERRAIN_NODE_H__
#define __RENDER_TERRAIN_NODE_H__

#include <string>
#include "../renderer.h"

/*
render node is responsible for control of which ogre scene entity
switching and attaching tiles to it

note, there is no hard dependancy on ogre
*/

namespace te
{
class render_terrain_hook;
class render_terrain_display_list;

//#############################################################################
class render_terrain_node
{
    friend class render_terrain_hook;
    friend class render_terrain_display_list;
    enum shadow
    {
        first, second
    };
public: //---------------------------------------------------------------------
    render_terrain_node(renderer& its_renderer) : my_renderer_(its_renderer)
{
    size_t x = te::terrain<tileData>::size_factor_;
    my_first_node_children_.resize(x*x); 
    my_second_node_children_.resize(x*x);

    my_renderer_.createTerrainNode ( my_first_node_name_ , my_first_node_children_ );
    my_renderer_.createTerrainNode ( my_second_node_name_ , my_second_node_children_ );
    isShadow_ = second;
    my_renderer_.attachTerrainNode ( my_first_node_name_ );
    my_hook_ = 0;
};
//-----------------------------------------------------------------------------
    /**
     *@remark
     *   get the non-rendered node name.
     *@return
     *   non-used node name, it is safe to do operations
     *   on it during the engine run
     */
    std::string& getShadow()
    {
        if (isShadow_ == first)
        {
            return my_first_node_name_;
        } else {
            return my_second_node_name_;
        }
    }
//-----------------------------------------------------------------------------
    /**
    *@remark
    *   get shadow node's list of child names
    */
    std::vector<std::string>& getShadowChildren()
    {
        if (isShadow_ == first)
        {
            return my_first_node_children_;
        } else {
            return my_second_node_children_;
        }

    }
//-----------------------------------------------------------------------------
    /**
     *@remark
     * render, this method should be called ONLY when the engine is NOT
     * rendering anything
    */
    void insertIntoScene()
    {
        if (isToBeRendered_)
        {
            if (isShadow_ == second)
            {
                my_renderer_.detachTerrainNode ( my_first_node_name_  );
                my_renderer_.attachTerrainNode ( my_second_node_name_ );
                isShadow_ = first;
            } else {
                my_renderer_.detachTerrainNode ( my_second_node_name_ );
                my_renderer_.attachTerrainNode ( my_first_node_name_  );
                isShadow_ = second;
            }
        }
    }
//-----------------------------------------------------------------------------
    /**
    * @remark destroy the underlying geometry of a node, done when a hook
    *         is destroyed. Note detaching and attaching should erase
    *         geometry of a node
    */
    void destroyGeometry()
    {
        if (isShadow_ == second)
        {
            my_renderer_.detachTerrainNode ( my_first_node_name_ );
            my_renderer_.attachTerrainNode ( my_first_node_name_ );
        } else {
            my_renderer_.detachTerrainNode ( my_second_node_name_ );
            my_renderer_.attachTerrainNode ( my_second_node_name_ );
        }
    }
//-----------------------------------------------------------------------------
    /**
     * @remark make this terrain node renderable
    */
    void makeRenderable()
    {
        isToBeRendered_ = true;
    }
//-----------------------------------------------------------------------------
    /**
     * @remark make this terrain node NOT renderable
     */
    void makeNonRenderable()
    {
        isToBeRendered_ = false;
    }
private: //--------------------------------------------------------------------
                         std::string my_first_node_name_;
                         std::string my_second_node_name_;

            std::vector<std::string> my_first_node_children_;
            std::vector<std::string> my_second_node_children_;

                              shadow isShadow_;                       
                           renderer& my_renderer_;
                render_terrain_hook* my_hook_;
                                bool isToBeRendered_;

                                 int mynum;
};
//#############################################################################
};


#endif