#ifndef  __RENDER_TILE_FACTORY__
#define  __RENDER_TILE_FACTORY__

#include "../renderer.h"
#include "../project_defaults.h"
#include "../map.h"
#include "Ogre.h"
#include <iostream>
#include <fstream>

namespace te
{
//#########################################################################
/**
 * @remark this class is responsible for 
 */
class render_tile_factory
{
public: //-----------------------------------------------------------------
    /**
    * @remark
    *   ctor - produces all possible tile meshes
    */
    render_tile_factory(Ogre::SceneManager* its_manager_)
    {
        my_manager_ = its_manager_;

        std::string meshPrefix("tile_");

        int meshNameNum = 0;

        std::cout << sizeof(Ogre::Mesh);

        std::cout << "creating tiles\n";

        for (int a = 0;a < 1;++a)
        {
            for (int b = 0;b < 16;++b)
            {
                for (int c = 0;c < 16;++c)
                {
                    for (int d = 0;d < 16;++d)
                    {
                        this->meshes[a][b][c][d] = meshPrefix + boost::lexical_cast<std::string>(meshNameNum);
                        this->makeTile(a, b, c, d,this->meshes[a][b][c][d]);
                        ++meshNameNum;
                    }
                }
            }
        }
    }

//-------------------------------------------------------------------------
    /**
    * @remark
    *   dtor
    */
    ~render_tile_factory()
    {
    }
//-------------------------------------------------------------------------
    /**
    * @remark
    *   get a meshPointer for a given tile type
    * @param
    *   tile to be retrieved
    * @return
    *   a pointer to the created mesh
    */
    std::string& getTile(te::tile& tile)
    {
        if (tile.first == 0)
        {
            return this->meshes[tile.first][tile.second][tile.third][tile.fourth];
        }
        if (tile.second == 0)
        {
            return this->meshes[tile.second][tile.third][tile.fourth][tile.first];
        }
        if (tile.third == 0)
        {
            return this->meshes[tile.third][tile.fourth][tile.first][tile.second];
        }
        if (tile.fourth == 0)
        {
            return this->meshes[tile.fourth][tile.first][tile.second][tile.third];
        }
        return meshes[0][0][0][0];
    }
private: //----------------------------------------------------------------
    /**
    * @remark
    *   build a simple tile mesh
    * @param
    *   vertice heights
    */
    void makeTile(int a, int b, int c, int d, std::string& meshName)
    {

        //find max value

        if (1) return;

        int max = te::maximal(a,te::maximal(b,te::maximal(c,d)));

        //  if a or c are max, then start triangle at A

        Ogre::MeshPtr fileptr;
        static Ogre::MeshSerializer meshser;

        // determine normals

        Ogre::Vector3 normal1 (0.0,absolute(a-b),32.0);
        Ogre::Vector3 tvec1   (32.0,absolute(a-c),32.0);
        normal1 = normal1.crossProduct (tvec1);

        Ogre::Vector3 normal2 (32.0,absolute(a-c),32.0);
        Ogre::Vector3 tvec2   (32.0,absolute(a-d),0.0);
        normal2 = normal2.crossProduct (tvec2);

        /*Ogre::Vector3 normal3 (32.0,absolute(b-c),0.0);
        Ogre::Vector3 tvec3   (32.0,absolute(b-d),-32.0);
        normal3 = normal3.crossProduct (tvec3);

        Ogre::Vector3 normal4 (32.0,absolute(b-d),-32.0);
        Ogre::Vector3 tvec4   (0.0,absolute(b-a),-32.0);
        normal2 = normal4.crossProduct (tvec4);*/

        Ogre::Vector3 nr1 = normal1 + normal2;
        //Ogre::Vector3 nr2 = normal1 + normal2 + normal2;

        nr1.normalise();
        //nr2.normalise();

        // build mesh

        Ogre::ManualObject* mnl = my_manager_->createManualObject("mesh");
        mnl->begin("te/Grass",Ogre::RenderOperation::OT_TRIANGLE_LIST);

        mnl->position    (0.0,a  ,0.0);
        mnl->colour      (1.0,1.0,1.0);
        mnl->normal      (nr1);
        mnl->textureCoord(0.0,0.0);

        mnl->position    (0.0,b  ,32.0);
        mnl->colour      (1.0,1.0,1.0);
        mnl->normal      (nr1);
        mnl->textureCoord(0.0,1.0);

        mnl->position    (32.0,c  ,32.0);
        mnl->colour      (1.0,1.0,1.0);
        mnl->normal      (nr1);
        mnl->textureCoord(1.0,1.0);

        mnl->triangle(0,1,2);

        mnl->position    (0.0,a  ,0.0);
        mnl->colour      (1.0,1.0,1.0);
        mnl->normal      (nr1);
        mnl->textureCoord(0.0,0.0);

        mnl->position    (32.0,c  ,32.0);
        mnl->colour      (1.0,1.0,1.0);
        mnl->normal      (nr1);
        mnl->textureCoord(1.0,1.0);

        mnl->position    (32.0,d  ,0.0);
        mnl->colour      (1.0,1.0,1.0);
        mnl->normal      (nr1);
        mnl->textureCoord(1.0,0.0);

        mnl->triangle(3,4,5);

        mnl->end();

        /*mnl->begin("te/Error",Ogre::RenderOperation::OT_LINE_LIST);

        mnl->position(16,16,16);
        mnl->position(normal1.x*100 + 16, normal1.y*100 + 16, normal1.z*100 + 16);
        
        mnl->index(0);
        mnl->index(1);

        mnl->end();*/

        fileptr = mnl->convertToMesh(meshName,Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME);

        meshser.exportMesh(&(*fileptr),"TEmedia/tiles/" + meshName + ".tile");

        my_manager_->destroyManualObject("mesh");
    }
//-------------------------------------------------------------------------
    Ogre::SceneManager* my_manager_;
    std::string meshes[1] [16][16][16];
};
//#########################################################################
};


#endif
