#ifndef __SERVER_ACTIONS_H__
#define __SERVER_ACTIONS_H__

#include "action_map.h"
#include "map.h"
#include "communication.h"

namespace te
{
//#############################################################################
// this map sets the map on a client to a given size, forcing the client to
// resize its image
class action_set_map_size : public action
{
public: //---------------------------------------------------------------------
    action_set_map_size (commLink& its_out_comm) : my_out_comm_(its_out_comm)
    {
        dgram_.params_.resize(0);
        dgram_.command_ = set_map_size;
    }
//-----------------------------------------------------------------------------
    ~action_set_map_size()
    {
    }
//-----------------------------------------------------------------------------
    void setSize(size_t longitude, size_t lattitude)
    {
        dgram_.params_.push_back(longitude);
        dgram_.params_.push_back(lattitude);
    }
//-----------------------------------------------------------------------------
    void invoke()
    {
        my_out_comm_.dispatch(dgram_);
        dgram_.params_.resize(0);
    }
private: //--------------------------------------------------------------------
    commLink& my_out_comm_;
    datagram dgram_;
};
//#############################################################################
class action_leaf_send : public action
{
public: //---------------------------------------------------------------------
    action_leaf_send (commLink& its_out_comm) : my_out_comm_(its_out_comm)
    {
        dgram_.params_.resize(0);
        dgram_.command_ = leaf_send;
    }
//-----------------------------------------------------------------------------
    ~action_leaf_send()
    {
    }
//-----------------------------------------------------------------------------
    void addTile(tileData& tile)
    {
        dgram_.params_.push_back(tile.msb);
        dgram_.params_.push_back(tile.lsb);
    }
//-----------------------------------------------------------------------------
    void setSize(size_t longitude, size_t lattitude)
    {
        dgram_.params_.push_back(longitude);
        dgram_.params_.push_back(lattitude);
    }
//-----------------------------------------------------------------------------
    void setVersion(size_t version)
    {
        dgram_.params_.push_back(version);
    }
//-----------------------------------------------------------------------------
    void invoke()
    {
        my_out_comm_.dispatch(dgram_);
        dgram_.params_.resize(0);
    }
private: //--------------------------------------------------------------------
    commLink& my_out_comm_;
    datagram dgram_;
};
//#############################################################################
class action_date_and_time : public action
{
public: //---------------------------------------------------------------------
    action_date_and_time(commLink& its_out_comm) : my_out_comm_(its_out_comm)
    {
        dgram_.params_.resize(0);
        dgram_.command_ = date_and_time;
    }
//-----------------------------------------------------------------------------
    void setData(size_t data)
    {
        dgram_.params_.push_back(data);
    }
//-----------------------------------------------------------------------------
    void invoke()
    {
        my_out_comm_.dispatch(dgram_);
        dgram_.params_.resize(0);
    }
private: //--------------------------------------------------------------------
    commLink& my_out_comm_;
    datagram dgram_;
};

};

#endif