#include "task.h"
using namespace ec;

block_task::block_task(size_t its_users, exco& its_list)\
    : user_count_(its_users), my_barrier_(static_cast<unsigned int>(its_users)), task(its_list)
{
    my_status_ = fresh;
}

//-----------------------------------------------------------------------------
block_task::block_task(const block_task& src, exco& its_list)\
    : my_barrier_(static_cast<unsigned int>(src.user_count_)),task(its_list)
{
    user_count_ = src.user_count_;
    my_status_ = src.my_status_;
}

//-----------------------------------------------------------------------------
block_task::~block_task()
{
}

//-----------------------------------------------------------------------------
void block_task::invoke(volatile size_t& finishCount)
{
    if(my_barrier_.wait())
    {
        if(my_status_ == obsolete)
        {
            ++finishCount;
            return;
        }

        if(attempt())
        {
            ++finishCount;
            logic();
            finalize();
        }
    }
    my_barrier_.wait();
    return;
}

//-----------------------------------------------------------------------------
bool block_task::isObsolete()
{
    if  (my_status_ == obsolete) return true;
    return false;
}
//-----------------------------------------------------------------------------
bool block_task::isFinished()
{
    if  (my_status_ == finished || isObsolete()) return true;
    return false;
}
//-----------------------------------------------------------------------------
bool block_task::attempt()
{
    if (my_status_ == fresh)
        return true;
    return false;
}
//-----------------------------------------------------------------------------
void block_task::finalize()
{
    if (my_status_ != obsolete)
        my_status_ = finished;
}
//-----------------------------------------------------------------------------
void block_task::refresh(bool also_obsolete)
{
    if (also_obsolete)
    {
        my_status_ = fresh;
        return;
    }

    if(my_status_ != obsolete)
    {
        my_status_ = fresh;
    }
}    
//-----------------------------------------------------------------------------
void block_task::makeObsolete()
{
    my_status_ = obsolete;
}
//#############################################################################
single_task::single_task(exco& its_list) : task(its_list),\
        my_status_(fresh)
{
    my_status_ = fresh;
}
//-----------------------------------------------------------------------------
single_task::single_task(const single_task& src,exco& its_list)\
    : task(its_list),\
        my_status_(fresh)
{
    my_status_ = src.my_status_;
}
//-----------------------------------------------------------------------------
single_task::~single_task()
{
}
//-----------------------------------------------------------------------------
void single_task::invoke(volatile size_t& finishCount)
{
    boost::try_mutex::scoped_try_lock entry_lock_(entry_mutex_,false);
    if (!entry_lock_.try_lock()) return;

    if(my_status_ == obsolete)
    {
        ++finishCount;
        return;
    }

    if (!attempt()) return;

    ++finishCount;
    this->logic();
    this->finalize();
}
//-----------------------------------------------------------------------------
bool single_task::isObsolete()
{
    if  (my_status_ == obsolete) return true;
    return false;
}
//-----------------------------------------------------------------------------
bool single_task::isFinished()
{
    if  (my_status_ == finished || isObsolete()) return true;
    return false;
}
//-----------------------------------------------------------------------------
bool single_task::attempt()
{
    if (my_status_ == fresh)
        return true;
    return false;
}

//-----------------------------------------------------------------------------
void single_task::finalize()
{
    if (my_status_ != obsolete)
        my_status_ = finished;
}

//-----------------------------------------------------------------------------
void single_task::refresh(bool also_obsolete)
{
    if (also_obsolete)
    {
        my_status_ = fresh;
        return;
    }

    if(my_status_ != obsolete)
    {
        my_status_ = fresh;
    }
} 
//-----------------------------------------------------------------------------
void single_task::makeObsolete()
{
    my_status_ = obsolete;
}
//#############################################################################
multi_task::multi_task(size_t its_users,exco& its_list)\
    : user_count_(its_users),\
        task(its_list),\
        my_status_(fresh)
{
    users_inside_ = 0;
        isLocked_ = false;
}
//-----------------------------------------------------------------------------
multi_task::multi_task(const multi_task& src,exco& its_list)\
    : user_count_(src.user_count_),\
        task(its_list),\
        my_status_(fresh)
{
    users_inside_ = 0;
    isLocked_ = src.isLocked_;
}
//-----------------------------------------------------------------------------
void multi_task::invoke(volatile size_t& finishCount)
{
    if(attempt())
    {
        boost::try_mutex::scoped_try_lock lk(my_mutex_,false);
        if (lk.try_lock() && !isLocked_)
        {
            isLocked_ = true;
            ++finishCount;
        }
        if (my_status_ == obsolete) return;
        ++users_inside_;
        logic();
        finalize();
        --users_inside_;
    }
}
//-----------------------------------------------------------------------------
bool multi_task::isObsolete()
{
    if (my_status_ == obsolete) return true;
    return false;
}
//-----------------------------------------------------------------------------
bool multi_task::attempt()
{
    return true;
}
//-----------------------------------------------------------------------------
void multi_task::finalize()
{
    if (my_status_ != obsolete)
        my_status_ = finished;
}
//-----------------------------------------------------------------------------
void multi_task::refresh(bool also_obsolete)
{
    isLocked_ = false;

    if (also_obsolete)
    {
        my_status_ = fresh;
        return;
    }

    if(my_status_ != obsolete)
    {
        my_status_ = fresh;
    }
}
//-----------------------------------------------------------------------------
void multi_task::makeObsolete()
{
    my_status_ = obsolete;
}
//-----------------------------------------------------------------------------
bool multi_task::isFinished()
{
    if  (my_status_ == finished || isObsolete()) return true;
    return false;
}
