#ifndef __TASK_H__
#define __TASK_H__

#include "boost/thread/mutex.hpp"
#include "boost/thread/barrier.hpp"
#include "boost/shared_ptr.hpp"
#include "exco.h"

namespace ec
{
//#############################################################################
enum taskStatus
{
    fresh, finished, obsolete, running
};

//#############################################################################
class block_task  : public task
{
public: //---------------------------------------------------------------------
    block_task(size_t its_users, exco& its_list);
//-----------------------------------------------------------------------------
    block_task(const block_task& src, exco& its_list);
//-----------------------------------------------------------------------------
    virtual ~block_task();
//-----------------------------------------------------------------------------
    virtual void invoke(volatile size_t& entryCount);
//-----------------------------------------------------------------------------
    virtual bool isObsolete();
//-----------------------------------------------------------------------------
    virtual bool isFinished();
protected: //------------------------------------------------------------------
    virtual void logic() = 0; // insert task logic here
//-----------------------------------------------------------------------------
    virtual bool attempt();
//-----------------------------------------------------------------------------
    virtual void finalize();
//-----------------------------------------------------------------------------
    virtual void refresh(bool also_obsolete);
//-----------------------------------------------------------------------------
    virtual void makeObsolete();
private: //--------------------------------------------------------------------
         boost::barrier my_barrier_;
    volatile taskStatus my_status_;
                 size_t user_count_;
};
//#############################################################################
class single_task : public task
{
public: //---------------------------------------------------------------------
    single_task(exco& its_list);
//-----------------------------------------------------------------------------
    single_task(const single_task& src,exco& its_list);
//-----------------------------------------------------------------------------
    virtual ~single_task();
//-----------------------------------------------------------------------------
    virtual void invoke(volatile size_t& entryCount);
//-----------------------------------------------------------------------------
    virtual bool isObsolete();
//-----------------------------------------------------------------------------
    virtual bool isFinished();
protected: //------------------------------------------------------------------
    virtual void logic() = 0; // insert task logic here
//-----------------------------------------------------------------------------
    virtual bool attempt();
//-----------------------------------------------------------------------------
    virtual void finalize();
//-----------------------------------------------------------------------------
    virtual void refresh(bool also_obsolete);
//-----------------------------------------------------------------------------
    virtual void makeObsolete();
private: //--------------------------------------------------------------------
        boost::try_mutex entry_mutex_;
     volatile taskStatus my_status_; 
//-----------------------------------------------------------------------------
};
//#############################################################################
class multi_task  : public task
{
public: //---------------------------------------------------------------------
    multi_task(size_t its_users,exco& its_list);
//-----------------------------------------------------------------------------
    multi_task(const multi_task& src,exco& its_list);
//-----------------------------------------------------------------------------
    virtual void invoke(volatile size_t& entryCount);
//-----------------------------------------------------------------------------
    virtual bool isObsolete();
protected: //------------------------------------------------------------------
    virtual void logic() = 0;
//-----------------------------------------------------------------------------
    virtual bool attempt();
//-----------------------------------------------------------------------------
    virtual void finalize();
//-----------------------------------------------------------------------------
    virtual void refresh(bool also_obsolete);
//-----------------------------------------------------------------------------
    virtual void makeObsolete();
//-----------------------------------------------------------------------------
    virtual bool isFinished();
private: //--------------------------------------------------------------------
             size_t user_count_;  // how many executors this mutli task will use
                                  // note, that there is no protection from over
                                  // population, this number is solely for the
                                  // exco you might want to use inside
         taskStatus my_status_;
    volatile size_t users_inside_;// amount of executors currently running the
                                  // task
   boost::try_mutex my_mutex_;
      volatile bool isLocked_;
};
//#############################################################################
};
#endif