/* 
timing_posix.h:  Wrapper for timing functions on Linux, MacOS, etc.

Copyright 2006, 2007 Adam Kadlubek
Copyright 2007 Nathanael Nerode

This file is part of TRANSPORT EMPIRE (TE).

TE is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

TE is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with TE; see the file COPYING.  If not, write to the Free
Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.  
*/

#ifndef __TIMING_H__
#define __TIMING_H__

#include  <iostream>
        
/* 
 * Windows Sleep and timeGetTime counts in milliseconds.
 * Linux offers us seconds, microseconds, or nanoseconds.
 * (sleep, usleep, and nanosleep respectively;
 * time, gettimeofday, and clock_gettime respectively)
 * Time can be got in milliseconds with ftime, but you *cannot*
 * sleep in milliseconds.
 */
#include <sys/time.h>
#include <time.h>

namespace te
{
//#########################################################################
    inline unsigned int getTicks()
    {
        // This code has overflow problems.  Rewrite calling code
        // to avoid dependency on millisecond resolution if possible --NCN

        struct timeval my_time; // deliberately uninitialized
        int result = gettimeofday(&my_time, NULL);
        if (result == -1)
        {
            perror("getTicks couldn't call gettimeofday");
            return 0;
	}
        return my_time.tv_sec * 1000 + my_time.tv_usec / 1000;
    };
//#########################################################################
    inline void pause(unsigned int howLong)    {
        // This code has overflow problems.  Rewrite calling code
        // to avoid dependency on millisecond resolution if possible --NCN
        usleep(howLong * 1000);
    };
//#########################################################################
    class runTime
    {
    public: //-------------------------------------------------------------
        runTime()
        {
            val = getTicks();
        }
    //---------------------------------------------------------------------
        unsigned int now() // call this to get the delay from last reset
                           // and reset the timer value
        {
            unsigned int retVal = getTicks() - val;
            val = getTicks();
            return  retVal;
        }
    //---------------------------------------------------------------------
        unsigned int later() // call this to get the delay from last reset
                             // when you don't want to reset the value
        {
            return  getTicks() - val;
        }
    //---------------------------------------------------------------------
    private:
        unsigned int val;
    };
};


#endif
